import logging

import psycopg2
import uuid
import os
import random
import string

from psc_orgkey import encode_orgkey

TEST_ORG_NAME = "ems-device-auth-poke-test-org.com"
POSTGRES_ENV = os.getenv('POSTGRES_ENV')

# This file could certainly be written better, but really it shouldn't require much maintenance or extension.


def get_devices(org_key, num_devices, dereg: False):
    # Use ENV vars
    conn = psycopg2.connect('')
    logging.debug("\nOpened database successfully")

    cur = conn.cursor()

    reg_suffix = ''
    if dereg:
        reg_suffix = 'IS_DEREG'
    else:
        reg_suffix = 'IS_REG'

    logging.debug("Trying to fetch existing devices...")
    cur.execute(f'''SELECT device_id, device_guid FROM device WHERE org_key = '{org_key}' AND name LIKE '%{reg_suffix}%' LIMIT {num_devices}''')
    rows = cur.fetchall()

    if not rows or len(rows) < num_devices:
        logging.debug("Devices do not exist, so creating ...")

        for i in range(num_devices - len(rows)):
            device_guid = uuid.uuid4().hex

            cur.execute(f'''INSERT INTO device(device_type_id, device_owner_id, device_guid, name, email, email_type_id, session_id, create_time, kill_switch_on, org_key, deployment_type, properties)
                               VALUES (1, null, '{device_guid}', 'Device 1 - {reg_suffix}', 'device@scargoinc.com', null, null, NOW(), false, '{org_key}', 0, null);''')

            logging.debug(f'\nInserted device {device_guid} successfully')

            conn.commit()
        conn.close()

        return get_devices(org_key, num_devices, dereg)
    else:
        logging.debug("Device exists, returning device")
        devices = []
        for row in rows:
            logging.debug(f'\ndevice_id {row[0]}')
            device_id = row[0]
            logging.debug(f'\ndevice_guid {row[1]}')
            device_guid = row[1]
            devices.append({
                "device_id": device_id,
                "device_guid": device_guid
            })

        conn.commit()
        conn.close()

        return devices


def get_org():
    # Use ENV vars
    conn = psycopg2.connect('')

    logging.debug("\nOpened database successfully")

    cur = conn.cursor()

    logging.debug("Trying to fetch existing org")

    cur.execute(f'''SELECT org_id, org_key FROM organization WHERE name = '{TEST_ORG_NAME}' ''')
    rows = cur.fetchall()

    if not rows:
        cur.execute(f'''INSERT INTO organization(org_type_id, name, admin_contact_id, billing_contact_id, registration_password, analytics_profile, analytics_version, storage_profile, create_time, mode, create_admin, kill_switch_on, ransomware_config, uninstall_code, license_endpoints, effective_th_endpoints, license_expires)
                        values (1,'{TEST_ORG_NAME}',null,null,'{''.join(random.choices(string.ascii_uppercase + string.digits, k=8))}','DOME','V2',null,NOW(),1,null,false,null,null,1,1,null)''')

        conn.commit()

        logging.debug(f'\nInserted org successfully')

        org = get_org()
        org_key = encode_orgkey(POSTGRES_ENV, org['org_id'])

        cur.execute(f'''UPDATE organization SET org_key = '{org_key}' WHERE org_id = {org['org_id']} ''')
        org['org_key'] = org_key

        conn.commit()
        conn.close()

        return org
    else:
        logging.debug("Org exists, returning org")
        org_id = 0
        org_key = ''
        for row in rows:
            logging.debug(f'\norg_id {row[0]}')
            org_id = row[0]
            logging.debug(f'\norg_key {row[1]}')
            org_key = row[1]

        conn.commit()
        conn.close()

        return {
            "org_id": org_id,
            "org_key": org_key
        }

def get_policy(devices: list, org_id):
    # Use ENV vars
    conn = psycopg2.connect('')

    logging.debug("\nOpened database successfully")

    cur = conn.cursor()

    logging.debug("Trying to fetch existing policy")

    cur.execute(f'SELECT device_group_id FROM device_group WHERE org_id = {org_id} LIMIT 1')
    rows = cur.fetchall()

    if not rows:
        cur.execute(f'''INSERT INTO device_group(org_id, policy_control, is_default, priority, name, priority_level)
                        VALUES ({org_id}, true, true, 100, 'default', 1)''')

        conn.commit()

        logging.debug(f'\nInserted policy successfully')

        return get_policy(devices, org_id)
    else:
        logging.debug("policy exists")
        policy_id = 0
        for row in rows:
            logging.debug(f'\npolicy_id {row[0]}')
            policy_id = row[0]

        for device in devices:
            cur.execute(f'''SELECT device_id FROM device_group_device WHERE device_id = {device['device_id']} AND device_group_id = {policy_id}''')
            rows = cur.fetchall()
            if not rows:
                cur.execute(f'''INSERT INTO device_group_device(device_id, device_group_id) VALUES({device['device_id']}, {policy_id})''')

            conn.commit()

        conn.commit()
        conn.close()

        return {
            "device_group_id": policy_id
        }
