#!/usr/bin/python3
"""
Created On: January 6th, 2020
Created By: Anthony Manno
Description: A PyTest Script to cover the API Endpoint testing for the Health Check.
    Health Check - HTTP GET
"""

import pytest
import requests


class TestHealthCheck:

    @pytest.fixture
    def supply_health_url(self, supply_base_url: str):
        """
        Provide the URL String for a basic Health Check.
        """
        return '{0}/health_check'.format(supply_base_url)

    def test_health_check_ok(self, supply_health_url: str):
        """
        Preform a standard Health check against the service.
        """
        health_response = requests.get(supply_health_url)
        assert health_response.status_code == 200, "Response was {0}, should be 200".format(health_response.status_code)

    def test_health_check_query(self, supply_health_url: str):
        """
        Verify the database is up and running.
        """
        health_response = requests.get('{0}?isReady=true'.format(supply_health_url))
        assert health_response.status_code == 200, "DB not ready - Status Code: {0}".format(health_response.status_code)
